@@ -1,6 +1,7 @@
-/* domajor.c (acdc) - copyright Mike Arnautov 1990-2018.
+/* domajor.c (acdc) - copyright Mike Arnautov 1990-2023.
  * Licensed under GPL, version 3 or later (see the supplied LICENCE file).
  *
+ * 08 Mar 22   MLA           Allowed for cpaces in game name.
  * 08 Aug 21   MLA           Replaced COMMA and SEMICOLON with barred names.
  * 30 Jul 21   MLA           Removed utf8 flag as no longer required.
  * 24 Jul 20   MLA           Use recase for recasing.
@@ -462,8 +463,17 @@ void domajor (void)
             gripe ("", "Repeated NAME directive.");
          strncpy (title, tp [1], sizeof (title) - 1);
          *(title + sizeof (title) - 1) = '\0';
+         {
+            char *cptr = title;
+            while ((cptr = strpbrk (cptr, " \t")))
+               *cptr++ = '_';
+         }
          break;
-
+         
+      case UTF8:
+         utf8 = 1;
+         break;
+         
       case GVERSION:
          if (*version)
             gripe ("", "Repeated VERSION directive.");
