@@ -1,7 +1,10 @@
-/* dominor.c (acdc) - copyright Mike Arnautov 1990-2020.
+/* dominor.c (acdc) - copyright Mike Arnautov 1990-2023.
  * Licensed under GPL, version 3 or later (see the supplied LICENCE file).
  *
- * 04 Jan 22   MLA           -debug now adds fputs calls instead of puts ones.
+ * 24 Jul 22   MLA           Bug: no STYLE macro but style variable!
+ * 03 Mar 22   MLA           Renamed DUMP to DUMPDATA.
+ * 28 Feb 22   MLA           -d now prints A-code lines being executed.
+ * 04 Jan 22   MLA           -d now adds fputs calls instead of puts ones.
  * 10 Nov 21   MLA           Added IFTURN.
  * 04 May 21   MLA           Added IFNEEDCMD.
  * 01 Jul 20   MLA           Allowed for style 2.
@@ -118,7 +121,7 @@ void addparam (int arg, char *name);
 int fndparam (char *name);
 int iniparam (int varsize);
 
-char cond_buf [160];
+char cond_buf [256];
 
 int entvalt (char *buf, int type, int what)
 {
@@ -138,6 +141,35 @@ int entvalv (char *buf, int type, int what)
   return len;
 }
 
+void show_line (char *tag)
+{
+   char *cptr = raw_line + strcspn(raw_line, "#\r\n") - 1;
+   while (*cptr == ' ' && *cptr == '\t' && cptr >= raw_line)
+      cptr--;
+   *(cptr + 1) = '\0';
+   
+   fprintf (code_file, "fputs(\"%s%s:%d \", stderr);\n",
+      tag, pathname[level], line_count [level]);
+   cptr = raw_line;
+   if (!(cptr = strchr (cptr, '"')))
+      fprintf (code_file, "fputs(\"%s\\n\", stderr);\n", raw_line);
+   else
+   {
+      char tbuf[256];
+      char *tptr;
+      strncpy (tbuf, raw_line, cptr - raw_line);
+      tptr = tbuf + (cptr-- - raw_line);
+      while (*(++cptr))
+      {
+        if (*cptr == '\\' || *cptr == '"')
+          *tptr++ = '\\';
+        *tptr++ = *cptr;
+      }
+      *tptr = '\0';
+      fprintf (code_file, "fputs(\"%s\\n\", stderr);\n", tbuf);
+   } 
+}
+
 void dominor (char *prochead, char *proccond)
 {
    int minor_type;
@@ -167,6 +199,9 @@ void dominor (char *prochead, char *proccond)
 
    char *cond_ptr = cond_buf;
 
+   if (debug && multiple_pending != ANYOF)
+      show_line ("*** ");
+
    memset (nestplus, '\0', sizeof (nestplus));
    while (1)
    {
@@ -217,12 +252,10 @@ void dominor (char *prochead, char *proccond)
       {
          locals = iniparam (flag_field_size[VARFLAG]/16 + 1);
          got_code = 1;
-         if (debug && prochead && *prochead)
-            fprintf (code_file, "fputs(\"*** %s ***\\n\", stderr);\n", prochead);
          if (proccond && *proccond)
             fprintf (code_file, "%s", proccond);
       }
-
+      
       if (minor_type >= ELSE || minor_type < NOT)
       {
          if (minor_type >= ELSE && (pending_not || pending_conj))
@@ -251,18 +284,19 @@ void dominor (char *prochead, char *proccond)
            gripe ("", "Another conditional expected.");
          else
            pending_otherwise = 0;
+
          if (minor_type < NOT)  /* It's an IF of sorts */
          {
-            if (pending_conj && *cond_buf)
+           if (pending_conj && *cond_buf)
               fprintf (code_file, "%s);\n", cond_buf);
-           fprintf (code_file, "value[0]");
-           if (pending_conj) fputc (pending_conj, code_file);
-           fputc ('=', code_file);
-           if (pending_not) fputc ('!', code_file);
-           fputc ('(', code_file);
+           strcpy (cond_buf, "value[0]");
+           cond_ptr = cond_buf + strlen(cond_buf);
+           if (pending_conj) *cond_ptr++ = pending_conj;
+           *cond_ptr++ = '=';
+           if (pending_not) *cond_ptr++ = '!';
+           *cond_ptr++ = '(';
 	   pending_if = 1;
-           cond_ptr = cond_buf;
-           *cond_buf = '\0';
+           *cond_ptr = '\0';
          }
          else
            pending_if = pending_conj = pending_not = 0;
@@ -278,8 +312,23 @@ void dominor (char *prochead, char *proccond)
          fputc ('\n', code_file);
          brace_pending = 0;
       }
+      if (debug && multiple_pending != ANYOF)
+         show_line ("");
 
       index = 0;
+      switch (minor_type)
+      {
+        case SAVE:
+        case DELETE:
+        case RESTORE:
+          index = 1;            
+          if ((ap[1] = fndsymb (TESTMINOR, tp [1])) == NULL)
+            gripe (tp[1], "Invalid minor directive modifier.");
+          break;
+        default:
+          break;
+      }
+
       while (tp [++index] != NULL)
       {
          chr = tp [index];
@@ -987,14 +1036,15 @@ void dominor (char *prochead, char *proccond)
                fprintf (code_file, "glue_text();\n");
          case QUIP:
          case SAY:
-         case VALUE:
          case DESCRIBE:
-            if (minor_type == VALUE)
+         case VALUE:
+            if (minor_type == VALUE && style < 12)
             {
                deprecate ("VALUE", 11, 0);
                type = 1;
             }
-            else if (minor_type == DESCRIBE)
+            else 
+            if (minor_type == DESCRIBE)
             {
                if (style >= 11)
                   type = 32;
@@ -1356,7 +1406,7 @@ void dominor (char *prochead, char *proccond)
                if (argtyp [1] != TEXT && argtyp [1] != VAR &&
                   argtyp [1] != LOCAL)
                   gripe (tp [1], "Not a text or variable.");
-               fprintf (code_file, "input(");
+               fprintf (code_file, "get_command(");
                if (argtyp [1] == VAR)
                   fprintf (code_file, "value[%d]);\n", argval [1]);
                else if (argtyp [1] == LOCAL)
@@ -1365,7 +1415,7 @@ void dominor (char *prochead, char *proccond)
                   fprintf (code_file, "%d);\n", argval [1]);
             }
             else
-               fprintf (code_file, "input(0);\n");
+               fprintf (code_file, "get_command(0);\n");
             fprintf (code_file, "if (loop) return(0);\n");
             break;
 
@@ -1565,7 +1615,7 @@ void dominor (char *prochead, char *proccond)
                gripe (tp [2], "Not a variable.");
             }
 
-            if (strcmp (tp[1], "memory") == 0)
+            if (ap[1]->refno == MEMORY)
             {
                if (minor_type == DELETE) type = 3;
                else if (minor_type == SAVE) type = 1;
@@ -1574,20 +1624,20 @@ void dominor (char *prochead, char *proccond)
                   (argtyp[2] == LOCAL) ? "lval" : "value", argval[2], type);
                break;
             }
-            else if (strcmp (tp[1], "command") == 0)
+            else if (ap[1]->refno == COMMAND)
             {
                fprintf (code_file, "input_store('%c', &%s[%d]);\n",
                  minor_type == SAVE ? 'S' : 'R',
                    argtyp [2] == LOCAL ? "lval" : "value", argval [2]);
                break;
             }
-            else if (strcmp (tp[1], "file") == 0)
+            else if (ap[1]->refno == SAVEFILE)
             {
                if (minor_type == SAVE) type = 1;
                else if (minor_type == RESTORE) type = 2;
                else type = 3;   /* Must be DELETE! */
             }
-            else if (strcmp (tp[1], "value") == 0)
+            else if (ap[1]->refno == VALUE && style >= 12)
             {
                if (minor_type == DELETE)
                   gripe ("VALUE", "Not a valid subtype of DELETE");
@@ -1596,7 +1646,7 @@ void dominor (char *prochead, char *proccond)
             }
             else
             {
-               gripe (tp[1], "Unknown save/restore operation type");
+               gripe (tp[1], "Unknown save/restore/delete operation type");
             }
             fprintf (code_file, "special(%d, &%s[%d]);\n",
                type, argtyp [2] == LOCAL ? "lval" : "value", argval [2]);
@@ -1644,7 +1694,7 @@ void dominor (char *prochead, char *proccond)
  *            break;
  */
 
-         case DUMP:
+         case DUMPDATA:
             fprintf (code_file, " show_data();");
             break;
 
@@ -1709,6 +1759,12 @@ void dominor (char *prochead, char *proccond)
                fprintf (code_file, ");\n");
             fprintf (code_file, "if (loop) return (0);\n");
             break;
+         
+         case MEMORY:
+         case SAVEFILE:
+         case COMMAND:
+            gripe (tp[0], "Naked minor directive modifier!");
+            break;
 
          default:
             gripe (tp [0], "not a known minor directive.");
