@@ -1,4 +1,4 @@
-/* organise.c (acdc) - copyright Mike Arnautov 1990-2022.
+/* organise.c (acdc) - copyright Mike Arnautov 1990-2023.
  * Licensed under GPL, version 3 or later (see the supplied LICENCE file).
  *
  *                           
@@ -833,7 +833,7 @@ void organise(void)
    fprintf (defs_file, "#define CENTRE_START 0%o\n", CENTRE_START);
    if (style > 10)
      fprintf (defs_file, "#define VHOLDER      0%o\n", VHOLDER);
-     fprintf (defs_file, "#define UTF8         %d\n",  1);
+     fprintf (defs_file, "#define UTF8         %d\n",  utf8);
    stage = 1;        /* Restore stage value */
    return;
 }
